#import "Connection.h"

#define DELTA 0.07
#define PI M_PI
#define HE_ARROW    4.0		// HEAD ARROW MID HEIGHT 
#define BA_ARROW   10.0		// HEAD ARROW BASE 
#define OX_ARROW  -7.0		// HEAD ARROW ORIGIN ON X
#define OY_ARROW    0.0		// HEAD ARROW ORIGIN ON Y

NSColor *colorC;		// a class variable???
NSBezierPath *hArrow;		// a class variable???

@implementation Connection 

// Class methods
+ (NSColor *)color
{
    return colorC;
}

+ (void)setColor: (NSColor *)aColor
{
   if (colorC==nil) colorC=[aColor retain]; 
   else if (![colorC isEqualTo:aColor]) { 
     [colorC autorelease];
     colorC = [aColor retain]; 
   }
}

+ (void)makeArrow
{
    NSPoint p; 
    p.x = OX_ARROW; p.y=OY_ARROW;
    hArrow = [[NSBezierPath bezierPath] retain];
    [hArrow moveToPoint: p];
    p.x = OX_ARROW - BA_ARROW;
    p.y = HE_ARROW;
    [hArrow lineToPoint: p];
    p.y = -p.y;
    [hArrow lineToPoint: p];
    [hArrow closePath];
}

+ (void)releaseHeadArrow
{
    [hArrow release]; 
}

// Instance Methods
- (id)initWithFrame:(NSRect)frameRect inView:(id)contentView
{
   self = [super initWithFrame:frameRect inView:contentView];
   startPoint = endPoint = [self frame].origin;
   startGraph=endGraph=nil; 
   weight = 1; 
   return self; 
}

- (void)dealloc
{
   [super dealloc]; 
}

- (void)reinitWithStartPointIn:(id) ngraph
{
   NSRect  fr=[self frame]; 
   startGraph = ngraph;
   fr.origin = [startGraph pointCenter]; 
   [self setFrame: fr];
}

- (NSBezierPath *)bezierPath
{
   NSBezierPath *drw; 
   
   startPoint = [startGraph pointCenter];
   drw = [NSBezierPath bezierPath]; 
   [drw setLineWidth:1.0];
   [drw moveToPoint:startPoint]; 
   if (endGraph!=nil) endPoint = [endGraph pointCenter];
   [drw lineToPoint:endPoint]; 

   return drw; 
}

- (void)drawFigure 
{
   NSBezierPath *drw = [self bezierPath]; 
   NSColor *tc = (selected)? [Figure color] : [Connection color]; 
   [tc set];
   [drw stroke];
   {
      NSBezierPath *tbp; 
      float dx, th; 
      NSAffineTransform *at=[NSAffineTransform transform]; 

      dx = startPoint.x - endPoint.x;
      th = (dx==0.0)? (PI/2.0):(atan((startPoint.y - endPoint.y)/dx));
      th = (startPoint.x > endPoint.x)?(PI+th):th;
      [at translateXBy:endPoint.x yBy: endPoint.y]; 
      [at rotateByRadians:th];
      tbp = [at transformBezierPath:hArrow];
      [tbp fill];
      [tbp stroke];
   }
}

- (BOOL)canResponseToEvent:(NSEvent *)theEvent
{
   NSRect r; 
   BOOL cr=NO; 
   NSPoint p=[[self superview] convertPoint:[theEvent locationInWindow] fromView:nil];

   r = NSMakeRect(MIN(startPoint.x,endPoint.x), MIN(startPoint.y,endPoint.y), 
                 fabs(endPoint.x-startPoint.x),fabs(endPoint.y-startPoint.y)); 
   if (NSPointInRect(p,r)) {
      double m1, m2; 
      if (endPoint.x == startPoint.x) 
         cr = (fabs(p.x-startPoint.x)<DELTA)?YES:NO; 
      else  {
         m1 = (endPoint.y-startPoint.y)/(endPoint.x-startPoint.x); 
         m2 = (p.y-startPoint.y)/(p.x-startPoint.x); 
         cr = (fabs(m1-m2)<DELTA)?YES:NO;
      }
   } 
   return cr; 
}


- (void)drawLabel
{
   NSMutableAttributedString *name; 
   NSPoint p = NSMakePoint((endPoint.x+startPoint.x)*0.5,(endPoint.y+startPoint.y)*0.5); 
   
   name = [[NSMutableAttributedString alloc] 
          initWithString:[[NSNumber numberWithInt:weight] stringValue]];
   [name addAttribute: NSFontAttributeName
                  value: [NSFont fontWithName:@"Times" size: 12.0]
                  range: NSMakeRange(0,[name length])];
   [name drawAtPoint: p];
   [name release]; 
}

- (void)mouseDown:(NSEvent *)theEvent
{
   [super mouseDown:theEvent];
   if (startGraph != nil) startPoint=[startGraph pointCenter]; 
}


- (void)mouseDragged:(NSEvent *)theEvent
{
   endPoint = [[self superview] convertPoint:[theEvent locationInWindow] fromView:nil]; 
   [[self superview] display];  
}

- (void)setWeight: (int)w
{
   if (w>0) weight=w;
}

- (int)weight
{
   return weight;
}

- (tGraph)typeGraph
{
   return ARC;
}

- (void)from:(id)fromGraph 
{
   startGraph = fromGraph;     
}

- (void)setEndGraph:(id)endG
{
   endGraph = endG; 
}

- (id)startGraph
{
   return startGraph; 
}

- (id)endGraph
{
   return endGraph; 
}

// Protocolo NSCoding
// arroy and tArrow variables are not sotred...  
- (void)encodeWithCoder:(NSCoder *)aCoder
{
    [super encodeWithCoder: aCoder];
    [aCoder encodeValuesOfObjCTypes: "ffff@@i",
            &startPoint.x, &startPoint.y, &endPoint.x, &endPoint.y,  
            &startGraph, &endGraph, &weight]; 
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
    self = [super initWithCoder: aDecoder];
    [aDecoder decodeValuesOfObjCTypes: "ffff@@i",
              &startPoint.x, &startPoint.y, &endPoint.x, &endPoint.y, 
              &startGraph, &endGraph, &weight]; 
    return self;
}

@end
