#import "Figure.h"
#import <Foundation/NSGeometry.h>

NSColor *colorS; 	// a class variable???? (color for selected objects)

@implementation Figure 

// Class Methods
+ (NSColor *)color
{
    return colorS;
}

+ (void)setColor: (NSColor *)aColor
{
   if (colorS==nil) colorS = [aColor retain];
   else if (![colorS isEqualTo:aColor]) {
      [colorS release];
      colorS = [aColor retain];
   }
}

// Instance Methods 

- (id)initWithFrame:(NSRect)frameRect inView:(id)contentView
{
   self = [super init]; 
   superView = contentView; 
   [self setFrame:(NSRect)frameRect];
   frameRect.origin = NSMakePoint(0.0, 0.0);
   figLabel = nil; 
   old = NSMakePoint(0.0, 0.0);
   selected = reference = NO; 
   return self; 
}

- (void)dealloc
{
   if (figLabel != nil)  [figLabel  release];
   [super dealloc];
}

- (NSRect)frame
{
    return frame;
}

- (void)setFrame:(NSRect)newFrame
{
    frame = newFrame; 
}

- (NSWindow *)window
{
    return [superView window];    
}

- (NSPoint)convertPoint:(NSPoint)punto fromView:(id)contentView
{
    NSPoint p; 
    p = [superView convertPoint:punto fromView:nil]; 
    p.x -= frame.origin.x; 
    p.y -= frame.origin.y; 
    return p; 
}

- (id)superview
{
    return superView;
}

- (void)setSuperview: (id)contentView
{
   superView = contentView; 
}

- (void)drawFigure
{
   return; 
}

- (void)drawLabel
{
   NSMutableAttributedString *name; 
   NSPoint p; 
   
   if (figLabel==nil) return; 

   p = NSMakePoint(frame.origin.x-2.0, frame.origin.y-15.0);
   name = [[NSMutableAttributedString alloc] initWithString:figLabel];
   [name addAttribute: NSFontAttributeName
                  value: [NSFont fontWithName:@"Times" size: 12.0]
                  range: NSMakeRange(0,[name length])];
   [name drawAtPoint: p];
   [name release]; 
}

- (NSBezierPath *)bezierPath
{
   return nil; 
}

- (void)setLabel: (NSString *)aLabel
{
   figLabel = aLabel; 
   [figLabel retain]; 
}

- (NSString *)label
{
   return figLabel;
}

/* Metodos para manejo de eventos */
- (NSPoint)deltaPositionTo:(NSEvent *)theEvent
{
    NSPoint d; 
    d = [[self superview] convertPoint:[theEvent locationInWindow] fromView:nil]; 
    d.x -= old.x; 
    d.y -= old.y; 
    return d; 
}

- (void)moveToRelativePoint: (NSPoint)d 
{
    NSRect fr= [self frame]; 
    fr.origin.x+=d.x; 
    fr.origin.y+=d.y; 
    if (reference) {
       old.x += d.x;
       old.y += d.y; 
    }
    [self setFrame: fr]; 
}

- (BOOL)canResponseToEvent:(NSEvent *)theEvent
{
    NSPoint p=[[self superview] convertPoint:[theEvent locationInWindow] fromView:nil]; 
    return (NSPointInRect(p, [self frame])?YES: NO); 
}

- (void)mouseDown:(NSEvent *)theEvent
{
    old = [[self superview] convertPoint:[theEvent locationInWindow] fromView:nil];
    reference = YES; 
}

- (void)mouseUp:(NSEvent *)theEvent
{
    old.x = old.y = 0.0;
    reference = NO; 
   [[self superview] setNeedsDisplay:YES]; 
}

- (void)mouseDragged:(NSEvent *)theEvent
{
    NSRect fr; 
    NSPoint d, new; 

    new = [[self superview] convertPoint:[theEvent locationInWindow] fromView:nil];
    fr = [self frame]; 
    d.x = new.x - old.x; 
    d.y = new.y - old.y; 
    fr.origin.x+=d.x; 
    fr.origin.y+=d.y; 
    old=new; 
    [self setFrame: fr]; 
    [[self superview] display]; 
}

- (tGraph)typeGraph
{
    return (tGraph)0; 
}

- (BOOL)isSelected
{
    return selected; 
}

- (void)setSelected:(BOOL)isel
{
    selected = isel; 
}

- (NSPoint) pointCenter
{
    NSPoint pc; 
    pc = frame.origin; 
    pc.x += frame.size.width/2; 
    pc.y += frame.size.height/2; 
    return pc; 
}

-(void) descripcion 
{
    printf("Descripcion\n"); 
    printf("FRAME: origin(%f,%f) size(%f,%f)\n",
           frame.origin.x,   frame.origin.y,
           frame.size.width, frame.size.height);
    printf("OLD: (%f, %f)\n", old.x, old.y);
}

// Protocolo NSCoding

// Las variables superView, old y selected no es necesario guardarlas...
// ?donde se inicializa el suerView??? 
- (void)encodeWithCoder:(NSCoder *)aCoder
{
    [aCoder encodeValuesOfObjCTypes: "ffff@", 
            &frame.origin.x,  &frame.origin.y,  &frame.size.width,  &frame.size.height,
            &figLabel];
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
    [aDecoder decodeValuesOfObjCTypes:"ffff@",
              &frame.origin.x,  &frame.origin.y,  &frame.size.width,  &frame.size.height,
              &figLabel];

    old.x = old.y = 0.0;
    selected = reference = NO; 
    [self description]; 
    return self;
}

@end
