#import "Matrix.h"

@implementation Matrix
- (id)init
{
    self = [super init];
    nRows = nColumns = 0; 
    current = base = nil; 
    return self; 
}

- (void)dealloc
{
   while(nColumns) [self deleteColumn: 0];   
   [super dealloc]; 
}

- (Element *)current
{
   return current; 
}

- (Element *)gotoI:(int) iI J:(int) iJ
{
   if (iI >= nRows || iJ >= nColumns) return nil; 

   current = [self bestFromForI:iI J:iJ]; 

   while([current posI] < iI)		// Avanza por renglon 
      current = [current nextI]; 

   while([current posI] > iI) 		// Retrocede por renglon  
      current = [current forwardI];   

   while([current posJ] < iJ) 		// Avanza por columa  
      current = [current nextJ]; 

   while([current posJ] > iJ) 		// Retrocede por columna  
      current = [current forwardJ]; 

   return current; 
}

- (Element *)bestFromForI:(int) iI J:(int)iJ; 
{
   int lbase, lcurrent; 

   lbase = abs(iI - [base posI]) + abs(iJ - [base posJ]); 
   lcurrent = abs(iI - [current posI]) + abs(iJ - [current posJ]);
   return (lbase>lcurrent)? current : base; 
}

- (Element *)home
{
   return current = base; 
}


- (Element *)findElementWithObject:(id)anObject
{
    Element *sl; // start line 
    BOOL c=NO; 

    sl = [self home]; 
    while(current!=nil) {
        if ([current content] !=nil ) {
           if ([[current content] isEqual:anObject]) {
              c = YES;
              break;
           } 
        }
        if ([current nextJ]!=nil) {
            current = [current nextJ]; 
        }
        else {
            current = [sl nextI]; 
            sl = current;             
        }
    }  
    if (!c) {
        [self home]; 
    } 

    return (c)?current:nil; 
}

// When de matrix is empty and request this method, 
// the matrix have one element, then nRows = Ncolumns = 1; 
- (void)addColumn 
{
   Element *temp, *offset, *tforwardI; 
   int i; 

   if (base == nil) {
      base = current = [[Element allocWithZone:[self zone]] init]; 
      [current setCoordI:0 J:0];
      nColumns = nRows = 1;
      return;      
   }
   tforwardI = nil; 
   offset = [self current]; 
   temp = [self gotoI:0 J:(nColumns-1)]; 
   for (i=0; i<nRows; i++) { 
       temp = [[Element allocWithZone:[self zone]] init];
       [current setNextJ: temp];
       [temp setForwardJ: current]; 
       if (i>0) { 
          [temp setForwardI: tforwardI]; 
          [tforwardI setNextI: temp]; 
       }
       [temp setCoordI: i J:nColumns]; 
       tforwardI = temp; 
       current = [current nextI]; 
      }
   nColumns++; 
   [self setCurrent:offset];
}

// When de matrix is empty and request this method,
// the matrix will have one element, so nRows = Ncolumns = 1;
- (void)addRow
{
   Element *temp, *offset, *tforward; 
   int j; 

   if (base == nil) {
      base = current = [[Element allocWithZone:[self zone]] init];
      [current setCoordI:0 J:0];
      nColumns = nRows = 1;
      return;
   }
   tforward = nil; 
   offset = [self current]; 
   temp = [self gotoI:(nRows-1) J:0]; 
   for (j=0; j<nColumns; j++) { 
       temp = [[Element alloc] init];
       [current setNextI: temp];
       [temp setForwardI: current]; 
       if (j>0) { 
          [temp setForwardJ: tforward]; 
          [tforward setNextJ: temp]; 
       }
       [temp setCoordI:nRows  J:j]; 
       tforward = temp; 
       current = [current nextJ]; 
   } 
   nRows++;
   [self setCurrent:offset];
}

- (void)deleteColumn: (int) iJ
{
   Element *temp, *tj; 

   temp = [self gotoI:0 J:iJ];
   while (temp!=nil) {
       if ( (tj= [temp forwardJ]) != nil ) {
           [tj setNextJ: [temp nextJ]];
       }
       if ( (tj=[temp nextJ]) != nil ) {
           [tj setForwardJ: [temp forwardJ]];
       }
       tj = [temp nextI]; 
       [temp release]; 
       temp = tj; 
   }
   if (nColumns == 1) {
      nColumns = nRows = 0; 
   }
   else {
      [self reenumerate];
      [self home];
      nColumns--; 
   }
}

- (void)deleteRow: (int) iI
{
   Element *temp, *tj;

   if (iI==0) return; 
   temp = [self gotoI:iI J:0];
   while (temp!=nil) {
      if ((tj=[temp forwardI])!=nil)
         [tj setNextI: [temp nextI]];
      if ( (tj=[temp nextI])!=nil)
         [tj setForwardI: [temp forwardI]];
      tj = [temp nextJ];
      [temp release];
      temp = tj;
   }
   if (nRows == 1) {
      nRows = nColumns = 0; 
   } else {
      [self reenumerate];
      [self home];
      nRows--;
   }
}

- (void)removeContentOfCurrent
{
    if ([current content]!= nil) {
        [[current content] release];
        [current setContent: nil]; 
    }
}

- (void)setCurrent: (Element *)c
{
   current = c;
}

- (void)reenumerate
{
   int i, j;
   id ti, tj;

   i = j = 0;
   ti = tj = [self home];
   while (ti!=nil)  {
       while (tj != nil) {
           [tj setCoordI:i J:j];
           tj = [tj nextJ];
           j++;
       }
       j = 0;
       i++;
       ti = [ti nextI];
       tj = ti; 
   }
}

- (int)rows
{
    return nRows; 
}

- (int)columns
{
    return nColumns;
}

// Metodos del protocolo NSCoding 

// El identificador <current> no se almacena pues se puede igualar cuando se carge nuevamente el objeto. 
- (void)encodeWithCoder:(NSCoder *)aCoder
{
   [aCoder encodeValuesOfObjCTypes:"ii@", &nRows, &nColumns, &base]; 
}

// El identificador <current> se le asigna su valor depues de decodificar al objeto 
// Matrix ya que es un valor temporal  
- (id)initWithCoder:(NSCoder *)aDecoder
{
   [aDecoder decodeValuesOfObjCTypes:"ii@", &nRows, &nColumns, &base]; 
   current = base;
   return self;
}

@end
