// 
//  PNController.m
//  PetrA
//
//  Created by Amilcar Meneses Viveros on Fri Jan 23 2004.
//  Copyright (c) 2004 CINVESTAV-IPN. All rights reserved.
//
//  Tambien se encarga de inicializar y liberar los colores de las clases 
//  Place, Transition, Connection y Figure; y crear la punta de la flecha 
//  de Connection.
//  Carga las preferencias del usuario

#import "PNController.h"
#import "PNDocument.h"
#import "Figure.h"
#import "Place.h"
#import "Connection.h"
#import "Transition.h"

NSString *className; 

@implementation PNController

static void deactivateKeyUIinToolWindow(NSMatrix *tools)
{
    NSArray *cellArray = [tools cells];
    NSEnumerator *enumerator = [cellArray objectEnumerator];
    NSCell *cell;
    while ((cell = [enumerator nextObject])) {
        [cell setRefusesFirstResponder:YES];
    }
}

+ (Class)currentGraphic
{
   return NSClassFromString(className);
}

- (void)awakeFromNib
{
   NSDictionary *dp=[[self preferencesFromDefaults] retain]; 

   [NSColorPanel sharedColorPanel];
   [Connection makeArrow];

   [Place setColor:      [NSUnarchiver unarchiveObjectWithData:[dp objectForKey:PlaceColor]]];
   [Transition setColor: [NSUnarchiver unarchiveObjectWithData:[dp objectForKey:TransitionColor]]];
   [Connection setColor: [NSUnarchiver unarchiveObjectWithData:[dp objectForKey:ConnectionColor]]];
   [Figure setColor:     [NSUnarchiver unarchiveObjectWithData:[dp objectForKey:SelectedColor]]];
   [colorTransition setColor: [Transition color]]; 
   [colorPlace setColor: [Place color]]; 
   [colorConnection setColor: [Connection color]]; 
   [colorSelected setColor: [Figure color]];
   [dp autorelease]; 
}

- (void)dealloc
{
   [[Place color] release]; 
   [[Transition color] release]; 
   [[Connection color] release]; 
   [[Figure color] release]; 
   [Connection releaseHeadArrow];
    if(className!=nil) [className release]; 
   [super dealloc]; 
}

- (IBAction)setCurrentGraphic:(id)sender
{
  if (className!=nil) [className autorelease]; 
  className = [[[[sender selectedCell] image] name] retain];
}

- (IBAction)setPreferences:(id)sender
{
   NSArray *documents = [[NSDocumentController sharedDocumentController] documents]; 
   unsigned count = (!documents)?0:[documents count]; 

   [Place setColor: [colorPlace color]]; 
   [Transition setColor: [colorTransition color]]; 
   [Connection setColor: [colorConnection color]]; 
   [Figure setColor: [colorSelected color]]; 

   while (count--) 
      [[documents objectAtIndex:count] updatePreferences];

   [self savePreferences]; 
}

// App's delegate methods 
// this code is from example TextEdit with 
// little changes for work with PetrA 
- (void)applicationWillFinishLaunching:(NSNotification *)notification
{
   NSPanel *toolWindow;
   deactivateKeyUIinToolWindow(palette);
   toolWindow = (NSPanel *)[palette window];
   [toolWindow setFrameAutosaveName:[toolWindow title]];
   [toolWindow setBecomesKeyOnlyIfNeeded:YES];
   [toolWindow setFloatingPanel:YES];
   [toolWindow orderFront:self];
}

// methods to deal with defaults 
- (NSDictionary *)preferencesFromDefaults {
   id temp; 
   NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
   NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithCapacity:5];

   temp = [defaults objectForKey:PlaceColor]; 
   [dict setObject: temp?temp:[NSArchiver archivedDataWithRootObject:[NSColor redColor]] forKey:PlaceColor];
   temp = [defaults objectForKey:TransitionColor]; 
   [dict setObject: temp?temp:[NSArchiver archivedDataWithRootObject:[NSColor redColor]] forKey:TransitionColor];
   temp = [defaults objectForKey:ConnectionColor]; 
   [dict setObject: temp?temp:[NSArchiver archivedDataWithRootObject:[NSColor blackColor]] forKey:ConnectionColor];
   temp = [defaults objectForKey:SelectedColor]; 
   [dict setObject: temp?temp:[NSArchiver archivedDataWithRootObject:[NSColor lightGrayColor]] forKey:SelectedColor];

   return dict; 
}

- (void)savePreferences
{
   NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];

   [defaults setObject:[NSArchiver archivedDataWithRootObject: [Place color]] 
             forKey:PlaceColor]; 
   [defaults setObject:[NSArchiver archivedDataWithRootObject: [Transition color]] 
             forKey:TransitionColor]; 
   [defaults setObject:[NSArchiver archivedDataWithRootObject: [Connection color]] 
             forKey:ConnectionColor]; 
   [defaults setObject:[NSArchiver archivedDataWithRootObject: [Figure color]] 
             forKey:SelectedColor]; 
}

@end
