#import "PNInspectorController.h"
#import "PNView.h"
#import "PNDocument.h"
#import "PNMatrix.h"
#import "Place.h"
#import "Transition.h"
#import "Connection.h" 

@implementation PNInspectorController

- (void)awakeFromNib
{
   [infConnection retain];
   [infPetriNet retain];
   [infPlace retain];
   [infTransition retain];

   [self setPagToView: infPetriNet];
   cInfo = PETRI_NET_INFO;
   [PNDocument setInspectorController: self]; 
   figure = nil;
   enable=NO; 
   [inspectorItem setEnabled:enable];
}

- (void)setPagToView: (id)theView
{
    [(NSBox *)multipleView setContentView: (NSView *)theView];
    [multipleView setNeedsDisplay:YES];
}

- (IBAction)enableLabels:(id)sender
{
   BOOL et=([figure canDisplayLabels])?NO:YES; 
   [figure setDisplayLabels:et]; 
   [figure setNeedsDisplay:YES]; 
}

- (IBAction)enableTransition:(id)sender
{
   BOOL et=([figure isEnable])?NO:YES; 
   [figure setEnable: et]; 
}

- (IBAction)setInfoConnection:(id)sender
{
   [figure setWeight: [weight intValue]]; 
   [[figure superview] setNeedsDisplay: YES]; 
}

- (IBAction)setInfoPlace:(id)sender
{
   [figure setTokens: [tokens intValue]]; 
   [figure setLabel: [labelPlace stringValue]]; 
   [[figure superview] setNeedsDisplay: YES]; 
}

- (IBAction)setInfoTransition:(id)sender
{
   [figure setLabel: [labelTransition stringValue]];
   [[figure superview] setNeedsDisplay: YES]; 
}

- (IBAction)displayInspectorPanel:(id)sender
{
   [inspectorPanel makeKeyAndOrderFront: nil];
}

// Display Methods 
- (void)displayPetriNetInfo: (id)anPNet
{
   figure=anPNet; 
   [places setIntValue:[[anPNet pnMatrix] countPlaces]];
   [enableLabels setState: ([anPNet canDisplayLabels])?NSOnState:NSOffState];
   [transitions setIntValue:[[anPNet pnMatrix] countTransitions]];
   if (cInfo!=PETRI_NET_INFO) {
      [self setPagToView: infPetriNet];
      cInfo = PETRI_NET_INFO;
   }
}

- (void)displayPlaceInfo:(id)anPlace
{
   figure=anPlace; 
   [tokens setIntValue:[(Place *)figure tokens]];
   [labelPlace setStringValue: [figure label]?[figure label]:@""];
   if (cInfo!=PLACE_INFO) {
      [self setPagToView: infPlace];
      cInfo = PLACE_INFO; 
   }
}

- (void)displayTransitionInfo:(id)anTransition
{
   figure=anTransition; 
   [labelTransition setStringValue: [figure label]?[figure label]:@""];
   [enableTransition setState: ([figure isEnable])?NSOnState:NSOffState]; 
   if (cInfo!=TRANSITION_INFO) {
      [self setPagToView: infTransition];
      cInfo = TRANSITION_INFO;
   }
}

- (void)displayConnectionInfo:(id)anConnection
{
   figure = anConnection; 
   [weight setIntValue: [(Connection *)figure weight]];
   if (cInfo!=CONNECTION_INFO) {
      [self setPagToView: infConnection];
      cInfo = CONNECTION_INFO;
   }
}

- (void)setEnableInspectorPanel: (BOOL)aFlag
{
   if (aFlag==NO) {
      [inspectorPanel performClose: self];
      enable=NO; 
   } else if (enable==NO) {
      [inspectorItem setEnabled:NO];
      enable=YES; 
   }
}

- (BOOL)validateMenuItem:(NSMenuItem *)anItem
{
   if ([[anItem title] isEqualToString:@"Inspector"]) 
      return enable; 
    return YES; 
}

@end
