#import "PNMatrix.h"
#import "Figure.h"
#import "Place.h"
#import "Transition.h"
#import "Connection.h"

@implementation PNMatrix
- (id)init
{
    self = [super init];
    [self addColumn]; 		// Create first matrix element
    nPlaces = nTransitions = 0; 
    dl = NO; 
    return self; 
}

- (unsigned) count
{
   return nTransitions+nPlaces+nLinks;
}

- (int) countPlaces 
{
   return nPlaces; 
}

- (int)countTransitions 
{
   return nTransitions; 
}

- (int)countLinks 
{
   return nLinks; 
}

- (void)addPlace: (id)newPlace
{
    nPlaces++; 
    [self addRow];
    [[self gotoI:nPlaces J:0] setContent: newPlace];
}

- (void)addTransition: (id)newTransition
{
   nTransitions++; 
   [self addColumn];
   [[self gotoI:0 J:nTransitions] setContent: newTransition]; 
}

- (void)deletePlaceInRow:(int)i
{
    if (i<1) return;
    [self deleteRow:i];
    nPlaces--;
}

- (void)deleteTransitionInColumn:(int)j
{
    if (j<1) return;
    [self deleteColumn:j];
    nTransitions--;
}


// Agregar una coneccion tiene tres posibilidades: 
// 1. Nueva coneccion 
// 2. Borra coneccion 
// 3. Incrementar el peso de la coneccion 
- (result_add)addConnection: (Connection *)newConnection; 
{
   Place *p; 
   Transition *t; 
   id oEqualPlace, oEqualTransition, oConnection;
   int lp, lt, i, j; 

   oEqualPlace = oEqualTransition = nil; 
   p = (Place *)([[newConnection startGraph] typeGraph] == PLACE)?
      [newConnection startGraph] : [newConnection endGraph]; 
   t = (Transition *)([[newConnection startGraph] typeGraph] != PLACE)?
      [newConnection startGraph] : [newConnection endGraph]; 

   // Find Place 
   lp = [self countPlaces]; 
   for (i=1; i<=lp && oEqualPlace==nil; i++) {
      oEqualPlace = [self gotoI:i J:0]; 
      oEqualPlace = ([[oEqualPlace content] isEqual:p])? oEqualPlace: nil;
   }

   // Find Transition 
   lt=[self countTransitions];
   for (j=1;  j<=lt && oEqualTransition==nil; j++) {
      oEqualTransition = [self gotoI:0 J:j];
      oEqualTransition = ([[oEqualTransition content] isEqual:t])?oEqualTransition:nil;
   }

   oConnection = [[self gotoI:--i J:--j] content];
   if (oConnection==nil) {
      [[self current] setContent: newConnection];
      nLinks++;
      return NEW_CONNECTION; 
   }
    else {
        int w;
        w = [oConnection weight];
        if ([[newConnection startGraph] typeGraph] == [[oConnection startGraph] typeGraph]) { 
            // Incrementa el peso
            [oConnection setWeight:w+1];
            return MODIFY_CONNECTION; 
        }
        else {
            [oConnection setWeight:w-1]; 
            if (w<=1) {   // se debe eliminar la coneccion 
               nLinks--;  // esto puede no ser consistente ya que se esta presuponiendo que 
                          // se va a eliminar la coneccion...
               return DELETE_CONNECTION; 
            }
            return MODIFY_CONNECTION; 
        }
    }
}

// Return a object "Place" that can response to the event "theEvent" 
- (id)placeCanResponseToEvent:(NSEvent *)theEvent
{
   int i, lim; 
   Element *tc; 
   id oResponse = nil; 

   tc = [self current];
   [self home];
   lim=[self countPlaces];
   for (i=1; i<=lim && oResponse == nil; i++) {
      oResponse = [[self gotoI:i J:0] content]; 
      oResponse = ([oResponse canResponseToEvent:theEvent])?oResponse:nil; 
   }
   [self setCurrent: tc];
   return oResponse; 
}

// Return a object "Transition" that can response to the event "theEvent" 
- (id)transitionCanResponseToEvent:(NSEvent *)theEvent
{
   int j, lim; 
   Element *tc; 
   id oResponse = nil; 

   tc = [self current];
   lim=[self countTransitions];

   for (j=1;  j<=lim && oResponse==nil; j++) {
      oResponse = [[self gotoI:0 J:j] content]; 
      oResponse = ([oResponse canResponseToEvent:theEvent])?oResponse:nil; 
   }
   [self setCurrent:tc];
   return oResponse; 
}

- (id)graphCanResponseToEvent:(NSEvent *)theEvent
{
    id resp = nil; 

    if ( (resp = [self transitionCanResponseToEvent:theEvent]) != nil) return resp;
    return [self placeCanResponseToEvent:theEvent]; 
}

- (void)runTransitions
{
    int j, lim;
    Element *tc;
    lim = [self countTransitions];
    tc = [self current];
    // Recurre los objetos transition
    for (j=1; j<=lim; j++)
        [(Transition *)[(Element *)[self gotoI:0 J:j] content] run];

    [self setCurrent: tc];
    return; 
}

// Protocolo NSCoding
- (void)encodeWithCoder:(NSCoder *)aCoder
{
   [super encodeWithCoder: aCoder];
   [aCoder encodeValuesOfObjCTypes: "iiii", &nPlaces, &nTransitions, &nLinks, &dl];
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
   self = [super initWithCoder: aDecoder];
   [aDecoder decodeValuesOfObjCTypes:"iiii", &nPlaces, &nTransitions, &nLinks, &dl];

   return self;
}

- (void)setDisplayLabels:(BOOL)aFlag
{
   dl = aFlag; 
}

- (BOOL)displayLabels
{
   return dl; 
}

@end
