#import "PNView.h"
#import "PNController.h"
#import "PNDocument.h"
#import "Place.h"
#import "Transition.h"
#import "Connection.h"

@implementation PNView 

- (id)initWithFrame:(NSRect)frameRect
{
   self = [super initWithFrame:frameRect];

   _graphics = [[NSMutableArray allocWithZone:[self zone]] init];
   _selectedGraphics = [[NSMutableArray allocWithZone:[self zone]] init]; 
   lastGraph = nil;
   factory = nil; 
//   dlabels = NO; 
   return self;
}

- (void)awakeFromNib
{
   if ([controller pnMatrix]!=nil) 
      [self setPNMatrix: [controller pnMatrix]];
   else 
      matrixC = [[PNMatrix allocWithZone:[self zone]] init]; 
   [[PNDocument inspectorController] displayPetriNetInfo: self]; 
}

- (void)dealloc
{
    if (running) {
        int i;
        running=NO;
        for (i=0; i<2000; i++); // do something while thread are stopped
    }
    [_graphics release];
    [_selectedGraphics release];
    if (!matrixC) [matrixC release]; 
    [super dealloc];
}

- (void)drawRect:(NSRect)rect
{
   int i,l;
   NSGraphicsContext *currentContext = [NSGraphicsContext currentContext];

   [[NSColor whiteColor] set]; 
   NSRectFill([self bounds]); 
   l = [_graphics count]; 
   for (i=0; i<l; i++) {
      [currentContext saveGraphicsState]; 
      [[_graphics objectAtIndex: i] drawFigure];
      if ([matrixC displayLabels]) 
         [[_graphics objectAtIndex: i] drawLabel];
      
      [currentContext restoreGraphicsState]; 
   }
}

- (BOOL)acceptsFirstMouse:(NSEvent *)theEvent {
    return YES;
}

- (void)mouseDown:(NSEvent *)event
{
   NSPoint mouseLoc = [self convertPoint:[event locationInWindow] fromView: nil];
   
   factory = [PNController currentGraphic];
   if (factory!=nil && !running) {
      id sg; 
      [[self window] setDocumentEdited: YES]; 
      NSRect fm = NSMakeRect(mouseLoc.x, mouseLoc.y, 5.0, 5.0); 
      lastGraph=[[factory allocWithZone:[self zone]] initWithFrame:fm inView:self];
      [self removeAllSelectedObjects];
      
      switch([lastGraph typeGraph]) {
      case PLACE: 
           [self addGraphic: lastGraph];
           [matrixC addPlace: lastGraph];
           [[PNDocument inspectorController] displayPetriNetInfo: self]; 
           break; 
      case TRANSITION:
           [self addGraphic: lastGraph];
           [matrixC addTransition: lastGraph]; 
           [[PNDocument inspectorController] displayPetriNetInfo: self]; 
           break; 
      case TOKEN:
           if ((sg=[matrixC placeCanResponseToEvent: event]) !=nil) {
              [sg addToken]; 
              [[PNDocument inspectorController] displayPlaceInfo: sg]; 
           } else 
              [[PNDocument inspectorController] displayPetriNetInfo: self]; 
           [lastGraph autorelease]; lastGraph=nil;
           break; 
      case ARC: 
           if ((sg=[matrixC graphCanResponseToEvent:event])!=nil) {
              [self addGraphicAtInit:lastGraph];
              [lastGraph reinitWithStartPointIn: sg];
           } else {
              [lastGraph release]; 
              lastGraph = nil; 
              [[PNDocument inspectorController] displayPetriNetInfo: self]; 
           }
           break;       
      }
   } else {
      int i; 
      BOOL found = NO; 
      BOOL shift = (([event modifierFlags] & NSShiftKeyMask)?YES:NO);
      lastGraph=nil; 
      for (i=[_graphics count]-1; i>=0 && !found; i--) {
         if ([[_graphics objectAtIndex:i] canResponseToEvent: event]) {
         lastGraph = [_graphics objectAtIndex:i];
            if ([lastGraph isSelected]) {
               if (shift) {
                  [self removeSelectedGraphic: lastGraph]; 
                  [lastGraph setSelected: NO]; 
                  lastGraph = nil; 
               } else [lastGraph mouseDown: event]; 
            } else { 
               if (!shift) {
                  [self removeAllSelectedObjects]; 
                  [self addSelectedGraphic: lastGraph];
                  [lastGraph mouseDown: event]; 
               } else {
                  [self addSelectedGraphic: lastGraph]; 
                  [lastGraph mouseDown: event]; 
               }
            }
            [self displayControlled];
            if ([lastGraph typeGraph]==ARC) lastGraph=nil;  
            found = YES; 
            [[self window] setDocumentEdited:YES]; 
         }
      }
      if (!found) {
         [[PNDocument inspectorController] displayPetriNetInfo: self];
         [self removeAllSelectedObjects];
      }
   }
   [self setNeedsDisplay: YES]; 
}

- (void)mouseUp: (NSEvent *)event
{
   if (lastGraph!=nil) {
      if([lastGraph typeGraph]==ARC) {
         id endObj=(([[lastGraph startGraph] typeGraph])==PLACE)? 
                    ([matrixC transitionCanResponseToEvent: event]) : 
                    ([matrixC placeCanResponseToEvent: event]); 
         if (endObj!=nil) {   // Coneccion  valida 
            [lastGraph setEndGraph: endObj]; 
            switch([matrixC addConnection: (Connection *)lastGraph]) {
            case NEW_CONNECTION: 
                 [[PNDocument inspectorController] displayConnectionInfo: lastGraph]; 
                 break; 
            case DELETE_CONNECTION:
                 [self removeGraphic: [[matrixC current] content]]; // esta el problema de nlink...
                 [matrixC removeContentOfCurrent]; 
                 [self removeGraphicAtInit];
                 [lastGraph release];
                 [[PNDocument inspectorController] displayPetriNetInfo: self];
                 break; 
            case MODIFY_CONNECTION: {
                 [self removeGraphicAtInit];
                 [lastGraph autorelease]; 
                 [[PNDocument inspectorController] displayConnectionInfo:[[matrixC current] content]];
                 }
                 break;
            } 
         } else { // Coneccion no valida 
            [self removeGraphicAtInit];
            [lastGraph release]; 
         }
       } else { 
         [lastGraph mouseUp: event]; 
       }
       lastGraph=nil; 
       [self setNeedsDisplay: YES]; 
   }
   factory=nil;
}

- (void)mouseDragged:(NSEvent *)event
{
   if (lastGraph!=nil) {
      if ([lastGraph typeGraph]==ARC && [lastGraph isSelected]==NO) {
         if (factory)
            [lastGraph mouseDragged: event]; 
      } else {
         NSPoint d = [lastGraph deltaPositionTo: event]; 
/* 
         if ([lastGraph typeGraph]==ARC) {
            if ([[lastGraph startGraph] isSelected]==NO) {
               [self addSelectedGraphic: [lastGraph startGraph]];
            } else if ([[lastGraph endGraph] isSelected]==NO) {
               [self addSelectedGraphic: [lastGraph endGraph]];            
            }
         }
*/
         [self moveAllSelectGraphicsToPoint: d];
         [self setNeedsDisplay: YES]; 
      }
   } 
}

// Sender-target method 
- (void)runPetriNet:(id)sender
{
   if (running) {
        running = NO; 
        [NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:1.0]];
    }
    else {
        running = YES;
        [[self window] setDocumentEdited:YES]; 
        [matrixC runTransitions]; 
    }
}

- (void)selectAll:(id)sender
{
   int i,l=[_graphics count];    
   [self removeAllSelectedObjects]; 
   for(i=0; i<l; i++) 
      [[_graphics objectAtIndex: i] setSelected: YES]; 

   [_selectedGraphics addObjectsFromArray:_graphics]; 
   [self setNeedsDisplay: YES];
}

- (void)cut:(id)sender
{
    int i,j, k;
    id osel, etem; 

    if ([_selectedGraphics count]>0) [[self window] setDocumentEdited:YES]; 
    while ([_selectedGraphics count]) {
        osel = [_selectedGraphics objectAtIndex:0];
        etem = [matrixC findElementWithObject: osel];
        switch([osel typeGraph]) {
        case PLACE:
              i = [etem posI];
              while(etem!=nil) {
                   osel = [etem content];
                   if (osel) {
                       [self removeGraphic:osel];
                       [self removeSelectedGraphic:osel];
                   }
                   etem = [etem nextJ]; 
              }
              [matrixC deletePlaceInRow:i]; 
              break; 
        case TRANSITION:
              k=0; 
              j = [etem posJ];
              while(etem!=nil) {
                   osel = [etem content];
                   if (osel) {
                       [self removeGraphic:osel];
                       [self removeSelectedGraphic:osel];
                   }
                   etem = [etem nextI];
              }
              [matrixC deleteTransitionInColumn: j]; 
              break;
        case ARC:
              [self removeGraphic:osel];
              [self removeSelectedGraphic:osel]; 
              [[etem content] release];
              [etem setContent: nil]; 
              break; 
        }
    }
    [[PNDocument inspectorController] displayPetriNetInfo: self];
    [self setNeedsDisplay:YES];
}

/* 
- (void)print:(id)sender
{
}
*/ 

/**************************************************************************/
/*                 Manejo de lista de objetos seleccionados               */ 
/**************************************************************************/
- (void)addGraphic:(id)aGraphic
{
    [_graphics insertObject:aGraphic atIndex:[_graphics count]]; 
}

- (void)addGraphicAtInit:(id)aGraphic
{
    [_graphics insertObject:aGraphic atIndex:0];
}

- (void)removeGraphicAtInit
{
    [_graphics removeObjectAtIndex:0]; 
}

- (void)removeGraphic:(id)aGraphic
{
    int i, c;
    c = [_graphics count];
    for (i=0; i<c; i++) {
        if ([[_graphics objectAtIndex:i] isEqual: aGraphic]) {
            [_graphics removeObjectAtIndex:i];
            i=c;
        }
    }
}

- (void)addSelectedGraphic: (id)aGraphic
{
    [aGraphic retain];
    [aGraphic setSelected:YES];

//    [_selectedGraphics insertObject:aGraphic atIndex:[_selectedGraphics count]];
    [_selectedGraphics addObject: aGraphic];
}

- (void)removeAllSelectedObjects
{
    int i; 
    for (i=0; i<[_selectedGraphics count]; i++) 
        [[_selectedGraphics objectAtIndex:i] setSelected:NO]; 
        
    [_selectedGraphics removeAllObjects];
}

- (void)removeSelectedGraphic:(id)aGraphic
{
    int i, c;
    c = [_selectedGraphics count];
    for (i=0; i<c; i++) {
        if ([[_selectedGraphics objectAtIndex:i] isEqual: aGraphic]) {
            [_selectedGraphics removeObjectAtIndex:i];
            return;
        }
    }
}

- (void) moveAllSelectGraphicsToPoint:(NSPoint) d
{
    id  tempGraph;
    Element *elementTempGraph; 
    int i; 
    int l=[_selectedGraphics count];

    for (i=0; i<l; i++) {
        tempGraph = [_selectedGraphics objectAtIndex:i]; 
        switch([tempGraph typeGraph]) {
        case ARC: break; 
        case PLACE: 
             elementTempGraph = [matrixC findElementWithObject: tempGraph]; 
             [tempGraph moveToRelativePoint:d];
             break; 
        case TRANSITION: 
             elementTempGraph = [matrixC findElementWithObject: tempGraph]; 
             [tempGraph moveToRelativePoint:d];
            break; 
        default: break; 
        } 
    }
}

- (id)pnMatrix
{
    return matrixC; 
}

- (void)setPNMatrix:(id)nMatrix
{
    int i,j, li, lj;
    id tmp; 

    if (matrixC!=nil) [matrixC autorelease]; 
    matrixC = nMatrix; 
    if (_graphics!= nil) [_graphics release];
    _graphics = [[NSMutableArray allocWithZone:[self zone]] init];
    li = [matrixC rows];
    lj = [matrixC columns];
    if (li==1 && lj==1) return;  

    // Recupera visualmente las ligas
    for (i=1; i<li; i++)
        for (j=1; j<lj; j++) {
            if ( (tmp=[[matrixC gotoI:i J:j] content]) != nil) {
               [self addGraphic: tmp];
               [tmp setSuperview: self]; 
            }
        }
    
    // Recupera visualmente los lugares 
    for (i=1; i<li; i++) {
        tmp = [[matrixC gotoI:i J:0] content]; 
        [self addGraphic: tmp];
        [tmp setSuperview: self]; 
    }

    // Recupera visualmente las transiciones 
    for (j=1; j<lj; j++) {
        tmp = [[matrixC gotoI:0 J:j] content]; 
        [self addGraphic: tmp];
        [tmp setSuperview: self]; 
    }
}

- (BOOL)isRunning
{
    return running;
}

- (void)displayControlled
{
    int l = [_selectedGraphics count]; 
    if (l==1) {
       switch([lastGraph typeGraph]) {
       case ARC: 
            [[PNDocument inspectorController] displayConnectionInfo:lastGraph]; 
            break;
       case PLACE:
            [[PNDocument inspectorController] displayPlaceInfo:lastGraph]; 
            break;
       case TRANSITION: 
            [[PNDocument inspectorController] displayTransitionInfo:lastGraph]; 
            break; 
       }
    }
    else [[PNDocument inspectorController] displayPetriNetInfo: self]; 
}

- (BOOL)canDisplayLabels 
{
   return [matrixC displayLabels];
}

- (void)setDisplayLabels:(BOOL)e 
{
   [matrixC setDisplayLabels:e];
   [[self window] setDocumentEdited: YES]; 
}

- (id) printableView
{
   if ([self isRunning]) // Para la ejecucion si la PN esta corriendo 
      [self runPetriNet: self]; 
   return self; 
}

@end
