#import "Transition.h"
#import "IOConnections.h"
#import "Connection.h"
#import "Place.h"
#import "PNView.h"


NSColor *colorT;
static NSLock *myLock;

@implementation Transition 

// Class Methods
+ (void)initialize
{
   myLock = [[NSLock alloc] init];
}

+ (NSColor *)color
{
    return colorT;
}

+ (void)setColor: (NSColor *)aColor
{
   if (colorT==nil) colorT = [aColor retain]; 
   else if (![colorT isEqualTo:aColor]) {
      [colorT autorelease]; 
      colorT = [aColor retain];
   }
}

- (id)initWithFrame:(NSRect)frameRect inView:(id)contentView 
{
   NSRect fr;

   [super initWithFrame:frameRect inView:contentView];
   active = YES;

   fr = [self frame];
   old = fr.origin; 
   fr.origin.x -= LWIDTH; 
   fr.origin.y -= LHEIGHT; 
   fr.size.width = LWIDTH+LWIDTH;
   fr.size.height = LHEIGHT+LHEIGHT;
   [self setFrame:fr]; 
   return self; 
}

- (void)dealloc
{
    [ioConnections release];
    [super dealloc];
}

- (NSBezierPath *)bezierPath
{
    NSBezierPath *drw; 
        
    drw = [NSBezierPath bezierPathWithRect:[self frame]];
    [drw setLineWidth:0.6]; 
    return drw; 
}

- (void)drawFigure
{
   NSBezierPath *drw=[self bezierPath];
   NSColor *tc = (selected)? [Figure color]: [Transition color]; 
   [tc set];
   [drw fill];
   [[NSColor blackColor] set];
   [drw stroke];
}


- (void)run
{
   if (active) {
      [self getConnection]; 
      [NSThread detachNewThreadSelector: @selector(execute:)
                toTarget: self withObject: ioConnections];
    }
}

- (void)execute: (id)connections
{
    NSAutoreleasePool *localPool = [[NSAutoreleasePool alloc] init];

    while ([superView isRunning]) {
        // begin critical section
        [myLock lock];
        if ([self inputsIsOK] && [superView isRunning]) {
            [self eliminateInputs];
            [self appendOutputs];
            [[self superview] display]; 
            [NSThread sleepUntilDate:[NSDate dateWithTimeIntervalSinceNow:0.9]];
        } 
        // end critical section
        [myLock unlock];
    }

    [localPool release];
    [NSThread exit];
    return;
}

- (void)eliminateInputs
{
    int i, l, t, w;
    Connection *cTemp;

    l = [[ioConnections inputs] count];
    for (i=0; i<l; i++) {
        cTemp = [[ioConnections inputs] objectAtIndex: i];
        w = [cTemp weight];
        t = [(Place *)[cTemp startGraph] tokens];
        [[cTemp startGraph] setTokens: t-w];
    }
}

- (void)appendOutputs
{
    int i, l, t, w;
    id cTemp;
    l = [[ioConnections outputs] count];
    for (i=0; i<l; i++) {
        cTemp = [[ioConnections outputs] objectAtIndex: i];
        w = [cTemp weight];
        t = [(Place *)[cTemp endGraph] tokens];
        [(Place *)[cTemp endGraph] setTokens: t+w];
        [[cTemp startGraph]  drawFigure];
    }
}

- (BOOL)inputsIsOK
{
    int i, l, t, w;
    id cTemp; 
    l = [[ioConnections inputs] count];

    for (i=0; i<l; i++) {
        cTemp = [[ioConnections inputs] objectAtIndex: i];
        w = [cTemp weight];
        t = [[cTemp startGraph] tokens];
        if (w>t) return NO; 
    }
    return YES; 
}

- (void)getConnection
{
    NSMutableArray *in,*out;
    id selfElement, temp; 
    
    in  = [[NSMutableArray allocWithZone:[self zone]] init];
    out = [[NSMutableArray allocWithZone:[self zone]] init];
    ioConnections = [[IOConnections alloc] init];

    selfElement = [[(PNView *)[self superview] pnMatrix] findElementWithObject: self];
    while ( [selfElement nextI]!=nil) {
        selfElement = [selfElement nextI];
        if ([selfElement content]!= nil) {
            temp = [selfElement content];
            [temp retain];
            if ([[temp startGraph] typeGraph]==TRANSITION)      // find outputs
                [out insertObject:temp atIndex:[out count]]; 
            else						// find inputs 
                [in insertObject:temp atIndex:[in count]];
        }
    }
    [ioConnections setInputs: in];
    [ioConnections setOutputs: out]; 
}

- (tGraph)typeGraph
{
    return TRANSITION;
}

-(void) descripcion
{
    [super descripcion]; 
}

- (BOOL)isEnable
{
   return active;
}
- (void)setEnable:(BOOL)e
{
   active = e; 
}

/*
- (BOOL)canRunTransition
{
    return canRunTransition; 
}

- (void)setCanRunTransition:(BOOL)ncrt
{
    canRunTransition = ncrt; 
}
*/

// Protocolo NSCoding
- (void)encodeWithCoder:(NSCoder *)aCoder
{
    [super encodeWithCoder: aCoder];
    [aCoder encodeValuesOfObjCTypes:"i", &active];
}

- (id)initWithCoder:(NSCoder *)aDecoder
{
    self = [super initWithCoder: aDecoder];
    [aDecoder decodeValuesOfObjCTypes:"i", &active]; 
//    canRunTransition = NO; 
    return self;
}

@end 